import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { CreateUserDto } from './dto/create-user.dto';
import { UpdateUserDto } from './dto/update-user.dto';
import { EntityNotFoundError, Repository } from 'typeorm';
import { User } from './entities/user.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { randomStringGenerator } from '@nestjs/common/utils/random-string-generator.util';
import { hashPassword } from '../helper/hash_password';
import { CoaService } from 'src/transaction/coa.service';
import { coaType } from 'src/helper/enum-list';

@Injectable()
export class UsersService {
  constructor(
    @InjectRepository(User)
    private usersRepository: Repository<User>,
    private coaService: CoaService
  ) {}

  async create(createUserDto: CreateUserDto) {
    const salt = randomStringGenerator();
    const result = await this.usersRepository.insert({
      username: createUserDto.username,
      password: await hashPassword(createUserDto.password, salt),
      salt,
    });

    return this.usersRepository.findOneOrFail(result.identifiers[0].id);
  }

  findAll(page:number) {
    return this.usersRepository.findAndCount({
      skip: page * 10,
      take: 10,
      order: {
        version: 'DESC',
      },
    });
  }

  findByRoles(relationId:string,page:number) {
    return this.usersRepository.findAndCount({
      skip: page * 10,
      take: 10,
      where:{
        roles:relationId
      },
      order: {
        updatedAt: 'DESC',
      },
    });
  }

  async findOne(id: string) {
    const coa = await this.coaService.findByUser(id,coaType.WALLET);
    try {
      const userData =  await this.usersRepository.findOneOrFail({ 
        where: { 
          id: id 
        },
        relations:["roles","superior"] 
      });
      return {
        ...userData,
        wallet:coa.amount,
      };
    } catch (e) {
      if (e instanceof EntityNotFoundError) {
        throw new HttpException(
          {
            statusCode: HttpStatus.NOT_FOUND,
            error: 'Data not found',
          },
          HttpStatus.NOT_FOUND,
        );
      } else {
        throw e;
      }
    }
  }

  async update(id: string, updateUserDto: UpdateUserDto) {
    try {
      await this.usersRepository.findOneOrFail(id);
    } catch (e) {
      if (e instanceof EntityNotFoundError) {
        throw new HttpException(
          {
            statusCode: HttpStatus.NOT_FOUND,
            error: 'Data not found',
          },
          HttpStatus.NOT_FOUND,
        );
      } else {
        throw e;
      }
    }

    // const result = await this.usersRepository.update(id, updateUserDto);

    return this.usersRepository.findOneOrFail(id);
  }

  async remove(id: string) {
    try {
      await this.usersRepository.findOneOrFail(id);
    } catch (e) {
      if (e instanceof EntityNotFoundError) {
        throw new HttpException(
          {
            statusCode: HttpStatus.NOT_FOUND,
            error: 'Data not found',
          },
          HttpStatus.NOT_FOUND,
        );
      } else {
        throw e;
      }
    }

    await this.usersRepository.delete(id);
  }

  async findOneByUsername(username: string) {
    return this.usersRepository.findOneOrFail({
      where: {
        username,
      },
    });
  }
}
